@@ -35,6 +35,7 @@ from pylint.checkers import BaseTokenChecker
 from pylint.checkers.utils import check_messages
 from pylint.utils import WarningScope, OPTION_RGX
 
+_CONTINUATION_BLOCK_OPENERS = ['elif', 'except', 'for', 'if', 'while', 'def', 'class']
 _KEYWORD_TOKENS = ['assert', 'del', 'elif', 'except', 'for', 'if', 'in', 'not',
                    'raise', 'return', 'while', 'yield']
 if sys.version_info < (3, 0):
@@ -45,8 +46,10 @@ _SPACED_OPERATORS = ['==', '<', '>', '!=', '<>', '<=', '>=',
                      '%=', '>>=', '<<=']
 _OPENING_BRACKETS = ['(', '[', '{']
 _CLOSING_BRACKETS = [')', ']', '}']
+_TAB_LENGTH = 8
 
 _EOL = frozenset([tokenize.NEWLINE, tokenize.NL, tokenize.COMMENT])
+_JUNK_TOKENS = (tokenize.COMMENT, tokenize.NL)
 
 # Whitespace checking policy constants
 _MUST = 0
@@ -77,6 +80,9 @@ MSGS = {
               'bad-indentation',
               'Used when an unexpected number of indentation\'s tabulations or '
               'spaces has been found.'),
+    'C0330': ('Wrong %s indentation%s.\n%s%s',
+              'bad-continuation',
+              'TODO'),
     'W0312': ('Found indentation with %ss instead of %ss',
               'mixed-indentation',
               'Used when there are some mixed tabs and spaces in a module.'),
@@ -138,6 +144,261 @@ def _column_distance(token1, token2):
     return token2[2][1] - token1[3][1]
 
 
+def _last_token_on_line_is(tokens, line_end, token):
+    return (
+        line_end > 0 and tokens.token(line_end-1) == token or
+        line_end > 1 and tokens.token(line_end-2) == token 
+        and tokens.type(line_end-1) == tokenize.COMMENT)
+
+
+def _token_followed_by_eol(tokens, position):
+  return (tokens.type(position+1) == tokenize.NL or
+          tokens.type(position+1) == tokenize.COMMENT and
+          tokens.type(position+2) == tokenize.NL)
+
+
+def _get_indent_length(line):
+  """Return the length of the indentation on the given token's line."""
+  result = 0
+  for char in line:
+    if char == ' ':
+      result += 1
+    elif char == '\t':
+      result += _TAB_LENGTH
+    else:
+      break
+  return result
+
+
+def _get_indent_hint_line(bar_positions, bad_position):
+    """Return a line with |s for each of the positions in the given lists."""
+    if not bar_positions:
+        return ''
+    markers = [(pos, '|') for pos in bar_positions]
+    markers.append((bad_position, '^'))
+    markers.sort()
+    line = [' '] * (markers[-1][0] + 1)
+    for position, marker in markers:
+        line[position] = marker
+    return ''.join(line)
+
+
+class _ContinuedIndent(object):
+    __slots__ = ('valid_outdent_offsets',
+                 'valid_continuation_offsets',
+                 'context_type',
+                 'token',
+                 'position')
+
+    def __init__(self,
+                 context_type,
+                 token,
+                 position,
+                 valid_outdent_offsets,
+                 valid_continuation_offsets):
+        self.valid_outdent_offsets = valid_outdent_offsets
+        self.valid_continuation_offsets = valid_continuation_offsets
+        self.context_type = context_type
+        self.position = position
+        self.token = token
+
+
+# The contexts for hanging indents.
+# A hanging indented dictionary value after :
+HANGING_DICT_VALUE = 'dict-value'
+# Hanging indentation in an expression.
+HANGING = 'hanging'
+# Hanging indentation in a block header.
+HANGING_BLOCK = 'hanging-block'
+# Continued indentation inside an expression.
+CONTINUED = 'continued'
+# Continued indentation in a block header.
+CONTINUED_BLOCK = 'continued-block'
+
+SINGLE_LINE = 'single'
+WITH_BODY = 'multi'
+
+_CONTINUATION_MSG_PARTS = {
+    HANGING_DICT_VALUE: ('hanging', ' in dict value'),
+    HANGING: ('hanging', ''),
+    HANGING_BLOCK: ('hanging', ' before block'),
+    CONTINUED: ('continued', ''),
+    CONTINUED_BLOCK: ('continued', ' before block'),
+}
+
+
+def _Offsets(*args):
+    """Valid indentation offsets for a continued line."""
+    return dict((a, None) for a in args)
+
+
+def _BeforeBlockOffsets(single, with_body):
+    """Valid alternative indent offsets for continued lines before blocks.
+
+    :param single: Valid offset for statements on a single logical line.
+    :param with_body: Valid offset for statements on several lines.
+    """
+    return {single: SINGLE_LINE, with_body: WITH_BODY}
+
+
+class TokenWrapper(object):
+    """A wrapper for readable access to token information."""
+
+    def __init__(self, tokens):
+        self._tokens = tokens
+
+    def token(self, idx):
+        return self._tokens[idx][1]
+
+    def type(self, idx):
+        return self._tokens[idx][0]
+
+    def start_line(self, idx):
+        return self._tokens[idx][2][0]
+
+    def start_col(self, idx):
+        return self._tokens[idx][2][1]
+
+    def line(self, idx):
+        return self._tokens[idx][4]
+
+
+class ContinuedLineState(object):
+    """Tracker for continued indentation inside a logical line."""
+
+    def __init__(self, tokens, config):
+        self._line_start = -1
+        self._cont_stack = []
+        self._is_block_opener = False
+        self.retained_warnings = []
+        self._config = config
+        self._tokens = TokenWrapper(tokens)
+
+    @property
+    def has_content(self):
+        return bool(self._cont_stack)
+
+    @property
+    def _block_indent_size(self):
+        return len(self._config.indent_string.replace('\t', ' ' * _TAB_LENGTH))
+
+    @property
+    def _continuation_size(self):
+        return self._config.indent_after_paren
+
+    def handle_line_start(self, pos):
+        """Record the first non-junk token at the start of a line."""
+        if self._line_start > -1:
+            return
+        self._is_block_opener = self._tokens.token(pos) in _CONTINUATION_BLOCK_OPENERS
+        self._line_start = pos
+
+    def next_physical_line(self):
+        """Prepares the tracker for a new physical line (NL)."""
+        self._line_start = -1
+        self._is_block_opener = False
+
+    def next_logical_line(self):
+        """Prepares the tracker for a new logical line (NEWLINE).
+
+        A new logical line only starts with block indentation.
+        """
+        self.next_physical_line()
+        self.retained_warnings = []
+        self._cont_stack = []
+
+    def add_block_warning(self, token_position, state, valid_offsets):
+        self.retained_warnings.append((token_position, state, valid_offsets))
+
+    def get_valid_offsets(self, idx):
+        """"Returns the valid offsets for the token at the given position."""
+        # The closing brace on a dict or the 'for' in a dict comprehension may
+        # reset two indent levels because the dict value is ended implicitly
+        stack_top = -1
+        if self._tokens.token(idx) in ('}', 'for') and self._cont_stack[-1].token == ':':
+            stack_top = -2
+        indent = self._cont_stack[stack_top]
+        if self._tokens.token(idx) in _CLOSING_BRACKETS:
+            valid_offsets = indent.valid_outdent_offsets
+        else:
+            valid_offsets = indent.valid_continuation_offsets
+        return indent, valid_offsets.copy()
+
+    def _hanging_indent_after_bracket(self, bracket, position):
+        """Extracts indentation information for a hanging indent."""
+        indentation = _get_indent_length(self._tokens.line(position))
+        if self._is_block_opener and self._continuation_size == self._block_indent_size:
+            return _ContinuedIndent(
+                HANGING_BLOCK,
+                bracket,
+                position,
+                _Offsets(indentation + self._continuation_size, indentation),
+                _BeforeBlockOffsets(indentation + self._continuation_size,
+                                    indentation + self._continuation_size * 2))
+        elif bracket == ':':
+            if self._cont_stack[-1].context_type == CONTINUED:
+                # If the dict key was on the same line as the open brace, the new
+                # correct indent should be relative to the key instead of the
+                # current indent level
+                paren_align = self._cont_stack[-1].valid_outdent_offsets
+                next_align = self._cont_stack[-1].valid_continuation_offsets.copy()
+                next_align[next_align.keys()[0] + self._continuation_size] = True
+            else:
+                next_align = _Offsets(indentation + self._continuation_size, indentation)
+                paren_align = _Offsets(indentation + self._continuation_size, indentation)
+            return _ContinuedIndent(HANGING_DICT_VALUE, bracket, position, paren_align, next_align)
+        else:
+            return _ContinuedIndent(
+                HANGING,
+                bracket,
+                position,
+                _Offsets(indentation, indentation + self._continuation_size),
+                _Offsets(indentation + self._continuation_size))
+
+    def _continuation_inside_bracket(self, bracket, pos):
+        """Extracts indentation information for a continued indent."""
+        indentation = _get_indent_length(self._tokens.line(pos))
+        if self._is_block_opener and self._tokens.start_col(pos+1) - indentation == self._block_indent_size:
+            return _ContinuedIndent(
+                CONTINUED_BLOCK,
+                bracket,
+                pos,
+                _Offsets(self._tokens.start_col(pos)),
+                _BeforeBlockOffsets(self._tokens.start_col(pos+1),
+                                    self._tokens.start_col(pos+1) + self._continuation_size))
+        else:
+            return _ContinuedIndent(
+                CONTINUED,
+                bracket,
+                pos,
+                _Offsets(self._tokens.start_col(pos)),
+                _Offsets(self._tokens.start_col(pos+1)))
+
+    def pop_token(self):
+        self._cont_stack.pop()
+
+    def push_token(self, token, position):
+        """Pushes a new token for continued indentation on the stack.
+
+        Tokens that can modify continued indentation offsets are:
+          * opening brackets
+          * 'lambda'
+          * : inside dictionaries
+
+        push_token relies on the caller to filter out those
+        interesting tokens.
+
+        :param token: The concrete token
+        :param position: The position of the token in the stream.
+        """
+        if _token_followed_by_eol(self._tokens, position):
+            self._cont_stack.append(
+                self._hanging_indent_after_bracket(token, position))
+        else:
+            self._cont_stack.append(
+                self._continuation_inside_bracket(token, position))
+
+
 class FormatChecker(BaseTokenChecker):
     """checks for :
     * unauthorized constructions
@@ -180,24 +441,41 @@ class FormatChecker(BaseTokenChecker):
                 {'default' : '    ', 'type' : "string", 'metavar' : '<string>',
                  'help' : 'String used as indentation unit. This is usually \
 "    " (4 spaces) or "\\t" (1 tab).'}),
+               ('indent-after-paren',
+                {'type': 'int', 'metavar': '<int>', 'default': 4,
+                 'help': 'Number of spaces of indent required inside a hanging '
+                         ' or continued line.'}),
                )
+
     def __init__(self, linter=None):
         BaseTokenChecker.__init__(self, linter)
         self._lines = None
         self._visited_lines = None
+        self._bracket_stack = [None]
 
-    def new_line(self, tok_type, line, line_num, junk):
+    def _pop_token(self):
+        self._bracket_stack.pop()
+        self._current_line.pop_token()
+
+    def _push_token(self, token, idx):
+        self._bracket_stack.append(token)
+        self._current_line.push_token(token, idx)
+
+    def new_line(self, tokens, line_end, line_start):
         """a new line has been encountered, process it if necessary"""
-        if not tok_type in junk:
+        if _last_token_on_line_is(tokens, line_end, ';'):
+            self.add_message('unnecessary-semicolon', line=tokens.start_line(line_end))
+
+        line_num = tokens.start_line(line_start)
+        line = tokens.line(line_start)
+        if tokens.type(line_start) not in _JUNK_TOKENS:
             self._lines[line_num] = line.split('\n')[0]
         self.check_lines(line, line_num)
 
     def process_module(self, module):
         self._keywords_with_parens = set()
-        for node in module.body:
-            if (isinstance(node, nodes.From) and node.modname == '__future__'
-                and any(name == 'print_function' for name, _ in node.names)):
-                self._keywords_with_parens.add('print')
+        if 'print_function' in module.future_imports:
+            self._keywords_with_parens.add('print')
 
     def _check_keyword_parentheses(self, tokens, start):
         """Check that there are not unnecessary parens after a keyword.
@@ -211,6 +489,8 @@ class FormatChecker(BaseTokenChecker):
         start: int; the position of the keyword in the token list.
         """
         # If the next token is not a paren, we're fine.
+        if self._inside_brackets(':') and tokens[start][1] == 'for':
+            self._pop_token()
         if tokens[start+1][1] != '(':
             return
 
@@ -270,7 +550,7 @@ class FormatChecker(BaseTokenChecker):
                     return
 
     def _opening_bracket(self, tokens, i):
-        self._bracket_stack.append(tokens[i][1])
+        self._push_token(tokens[i][1], i)
         # Special case: ignore slices
         if tokens[i][1] == '[' and tokens[i+1][1] == ':':
             return
@@ -283,7 +563,9 @@ class FormatChecker(BaseTokenChecker):
             self._check_space(tokens, i, (_IGNORE, _MUST_NOT))
 
     def _closing_bracket(self, tokens, i):
-        self._bracket_stack.pop()
+        if self._inside_brackets(':'):
+            self._pop_token()
+        self._pop_token()
         # Special case: ignore slices
         if tokens[i-1][1] == ':' and tokens[i][1] == ']':
             return
@@ -302,7 +584,7 @@ class FormatChecker(BaseTokenChecker):
             self._check_space(tokens, i, (_MUST, _MUST))
 
     def _open_lambda(self, tokens, i): # pylint:disable=unused-argument
-        self._bracket_stack.append('lambda')
+        self._push_token('lambda', i)
 
     def _handle_colon(self, tokens, i):
         # Special case: ignore slices
@@ -316,7 +598,9 @@ class FormatChecker(BaseTokenChecker):
         self._check_space(tokens, i, policy)
 
         if self._inside_brackets('lambda'):
-            self._bracket_stack.pop()
+            self._pop_token()
+        elif self._inside_brackets('{'):
+            self._push_token(':', i)
 
     def _handle_comma(self, tokens, i):
         # Only require a following whitespace if this is
@@ -325,6 +609,8 @@ class FormatChecker(BaseTokenChecker):
             self._check_space(tokens, i, (_MUST_NOT, _IGNORE))
         else:
             self._check_space(tokens, i, (_MUST_NOT, _MUST))
+        if self._inside_brackets(':'):
+            self._pop_token()
 
     def _check_surrounded_by_space(self, tokens, i):
         """Check that a binary operator is surrounded by exactly one space."""
@@ -383,6 +669,10 @@ class FormatChecker(BaseTokenChecker):
     def _inside_brackets(self, left):
         return self._bracket_stack[-1] == left
 
+    def _handle_old_ne_operator(self, tokens, i):
+        if tokens[i][1] == '<>':
+            self.add_message('old-ne-operator', line=tokens[i][2][0])
+
     def _prepare_token_dispatcher(self):
         raw = [
             (_KEYWORD_TOKENS,
@@ -401,6 +691,8 @@ class FormatChecker(BaseTokenChecker):
             ([':'], self._handle_colon),
 
             (['lambda'], self._open_lambda),
+
+            (['<>'], self._handle_old_ne_operator),
             ]
 
         dispatch = {}
@@ -419,76 +711,63 @@ class FormatChecker(BaseTokenChecker):
            regular expression).
         """
         self._bracket_stack = [None]
-        indent = tokenize.INDENT
-        dedent = tokenize.DEDENT
-        newline = tokenize.NEWLINE
-        junk = (tokenize.COMMENT, tokenize.NL)
         indents = [0]
-        check_equal = 0
+        check_equal = False
         line_num = 0
-        previous = None
         self._lines = {}
         self._visited_lines = {}
-        new_line_delay = False
         token_handlers = self._prepare_token_dispatcher()
+
+        self._current_line = ContinuedLineState(tokens, self.config)
         for idx, (tok_type, token, start, _, line) in enumerate(tokens):
-            if new_line_delay:
-                new_line_delay = False
-                self.new_line(tok_type, line, line_num, junk)
             if start[0] != line_num:
-                if previous is not None and previous[0] == tokenize.OP and previous[1] == ';':
-                    self.add_message('unnecessary-semicolon', line=previous[2])
-                previous = None
                 line_num = start[0]
                 # A tokenizer oddity: if an indented line contains a multi-line
                 # docstring, the line member of the INDENT token does not contain
-                # the full line; therefore we delay checking the new line until
-                # the next token.
+                # the full line; therefore we check the next token on the line.
                 if tok_type == tokenize.INDENT:
-                    new_line_delay = True
+                    self.new_line(TokenWrapper(tokens), idx-1, idx+1)
                 else:
-                    self.new_line(tok_type, line, line_num, junk)
-            if tok_type not in (indent, dedent, newline) + junk:
-                previous = tok_type, token, start[0]
-
-            if tok_type == tokenize.OP:
-                if token == '<>':
-                    self.add_message('old-ne-operator', line=line_num)
-            elif tok_type == tokenize.NUMBER:
-                if token.endswith('l'):
-                    self.add_message('lowercase-l-suffix', line=line_num)
-
-            elif tok_type == newline:
+                    self.new_line(TokenWrapper(tokens), idx-1, idx)
+            
+            if tok_type == tokenize.NEWLINE:
                 # a program statement, or ENDMARKER, will eventually follow,
                 # after some (possibly empty) run of tokens of the form
                 #     (NL | COMMENT)* (INDENT | DEDENT+)?
                 # If an INDENT appears, setting check_equal is wrong, and will
                 # be undone when we see the INDENT.
-                check_equal = 1
-
-            elif tok_type == indent:
-                check_equal = 0
+                check_equal = True
+                self._process_retained_warnings(TokenWrapper(tokens), idx)
+                self._current_line.next_logical_line()
+            elif tok_type == tokenize.INDENT:
+                check_equal = False
                 self.check_indent_level(token, indents[-1]+1, line_num)
                 indents.append(indents[-1]+1)
-
-            elif tok_type == dedent:
+            elif tok_type == tokenize.DEDENT:
                 # there's nothing we need to check here!  what's important is
                 # that when the run of DEDENTs ends, the indentation of the
                 # program statement (or ENDMARKER) that triggered the run is
                 # equal to what's left at the top of the indents stack
-                check_equal = 1
+                check_equal = True
                 if len(indents) > 1:
                     del indents[-1]
-
-            elif check_equal and tok_type not in junk:
-                # this is the first "real token" following a NEWLINE, so it
+            elif tok_type == tokenize.NL:
+                self._check_continued_indentation(TokenWrapper(tokens), idx+1)
+                self._current_line.next_physical_line()
+            elif tok_type != tokenize.COMMENT:
+                self._current_line.handle_line_start(idx)
+                # This is the first concrete token following a NEWLINE, so it
                 # must be the first token of the next program statement, or an
                 # ENDMARKER; the "line" argument exposes the leading whitespace
                 # for this statement; in the case of ENDMARKER, line is an empty
                 # string, so will properly match the empty string with which the
                 # "indents" stack was seeded
-                check_equal = 0
-                self.check_indent_level(line, indents[-1], line_num)
+                if check_equal:
+                    check_equal = False
+                    self.check_indent_level(line, indents[-1], line_num)
+
+            if tok_type == tokenize.NUMBER and token.endswith('l'):
+                self.add_message('lowercase-l-suffix', line=line_num)
 
             try:
                 handler = token_handlers[token]
@@ -501,6 +780,49 @@ class FormatChecker(BaseTokenChecker):
         if line_num > self.config.max_module_lines:
             self.add_message('too-many-lines', args=line_num, line=1)
 
+    def _process_retained_warnings(self, tokens, current_pos):
+        single_line_block_stmt = not _last_token_on_line_is(tokens, current_pos, ':')
+
+        for indent_pos, state, offsets in self._current_line.retained_warnings:
+            block_type = offsets[tokens.start_col(indent_pos)]
+            hints = dict((k, v) for k, v in offsets.iteritems()
+                         if v != block_type)
+            if single_line_block_stmt and block_type == WITH_BODY:
+                self._add_continuation_message(state, hints, tokens, indent_pos)
+            elif not single_line_block_stmt and block_type == SINGLE_LINE:
+                self._add_continuation_message(state, hints, tokens, indent_pos)
+
+    def _check_continued_indentation(self, tokens, next_idx):
+        # Do not issue any warnings if the next line is empty.
+        if not self._current_line.has_content or tokens.type(next_idx) == tokenize.NL:
+            return
+
+        state, valid_offsets = self._current_line.get_valid_offsets(next_idx)
+        # Special handling for hanging comments. If the last line ended with a
+        # comment and the new line contains only a comment, the line may also be
+        # indented to the start of the previous comment.
+        if (tokens.type(next_idx) == tokenize.COMMENT and
+                tokens.type(next_idx-2) == tokenize.COMMENT):
+            valid_offsets[tokens.start_col(next_idx-2)] = True
+
+        # We can only decide if the indentation of a continued line before opening
+        # a new block is valid once we know of the body of the block is on the
+        # same line as the block opener. Since the token processing is single-pass,
+        # emitting those warnings is delayed until the block opener is processed.
+        if (state.context_type in (HANGING_BLOCK, CONTINUED_BLOCK)
+                and tokens.start_col(next_idx) in valid_offsets):
+            self._current_line.add_block_warning(next_idx, state, valid_offsets)
+        elif tokens.start_col(next_idx) not in valid_offsets:
+            self._add_continuation_message(state, valid_offsets, tokens, next_idx)
+
+    def _add_continuation_message(self, state, offsets, tokens, position):
+        readable_type, readable_position = _CONTINUATION_MSG_PARTS[state.context_type]
+        hint_line = _get_indent_hint_line(offsets, tokens.start_col(position))
+        self.add_message(
+            'bad-continuation',
+            line=tokens.start_line(position),
+            args=(readable_type, readable_position, tokens.line(position), hint_line))
+
     @check_messages('multiple-statements')
     def visit_default(self, node):
         """check the node line number and check it if not yet done"""
