@@ -39,7 +39,7 @@ from logilab.common.configuration import UnsupportedAction, OptionsManagerMixIn
 from logilab.common.optik_ext import check_csv
 from logilab.common.modutils import load_module_from_name, get_module_part
 from logilab.common.interface import implements
-from logilab.common.textutils import splitstrip
+from logilab.common.textutils import splitstrip, unquote
 from logilab.common.ureports import Table, Text, Section
 from logilab.common.__pkginfo__ import version as common_version
 
@@ -607,7 +607,7 @@ warning, statement which respectively contain the number of errors / warnings\
         # build ast and check modules or packages
         for descr in self.expand_files(files_or_modules):
             modname, filepath = descr['name'], descr['path']
-            if not self.should_analyze_file(modname, filepath):
+            if not descr['isarg'] and not self.should_analyze_file(modname, filepath):
                 continue
             if self.config.files_output:
                 reportfile = 'pylint_%s.%s' % (modname, self.reporter.extension)
@@ -907,7 +907,7 @@ group are mutually exclusive.'),
         try:
             preprocess_options(args, {
                     # option: (callback, takearg)
-                    'init-hooks':   (cb_init_hook, True),
+                    'init-hook':   (cb_init_hook, True),
                     'rcfile':       (self.cb_set_rcfile, True),
                     'load-plugins': (self.cb_add_plugins, True),
                     })
@@ -1009,8 +1009,12 @@ are done by default'''}),
         linter.disable('suppressed-message')
         linter.disable('useless-suppression')
         linter.read_config_file()
-        # is there some additional plugins in the file configuration, in
         config_parser = linter.cfgfile_parser
+        # run init hook, if present, before loading plugins
+        if config_parser.has_option('MASTER', 'init-hook'):
+            cb_init_hook('init-hook',
+                         unquote(config_parser.get('MASTER', 'init-hook')))
+        # is there some additional plugins in the file configuration, in
         if config_parser.has_option('MASTER', 'load-plugins'):
             plugins = splitstrip(config_parser.get('MASTER', 'load-plugins'))
             linter.load_plugin_modules(plugins)
