@@ -10,7 +10,7 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """utilities methods and classes for reporters"""
 
 import sys
@@ -28,6 +28,10 @@ if sys.version_info >= (3, 0):
     def cmp(a, b):
         return (a > b) - (a < b)
 
+if sys.version_info < (2, 6):
+    import stringformat
+    stringformat.init(True)
+
 def diff_string(old, new):
     """given a old and new int value, return a string representing the
     difference
