@@ -3,7 +3,7 @@
 import sys
 
 from pylint.interfaces import IReporter
-from pylint.reporters import BaseReporter
+from pylint.reporters import BaseReporter, Message
 from logilab.common.ureports import TextWriter
 
 
@@ -16,16 +16,12 @@ class GUIReporter(BaseReporter):
     def __init__(self, gui, output=sys.stdout):
         """init"""
         BaseReporter.__init__(self, output)
-        self.msgs = []
         self.gui = gui
 
     def add_message(self, msg_id, location, msg):
         """manage message of different type and in the context of path"""
-        filename, module, obj, line, col_offset = location
-        msg = Message(self, msg_id, location, msg)
-        full_msg = [msg.C, msg_id, filename, module, obj, str(line), msg]
-        self.msgs += [[sigle, module, obj, str(line)]]
-        self.gui.msg_queue.put(full_msg)
+        message = Message(self, msg_id, location, msg)
+        self.gui.msg_queue.put(message)
 
     def _display(self, layout):
         """launch layouts display"""
