@@ -279,8 +279,9 @@ class PyLinterTC(TestCase):
         self.linter.set_reporter(text.TextReporter())
         self.linter.config.files_output = True
         self.linter.should_analyze_file = lambda *args: False
-        self.linter.check('os')
-        self.assertFalse(os.path.exists('pylint_os.txt'))
+        self.linter.check('logilab')
+        self.assertTrue(os.path.exists('pylint_logilab.txt'))
+        self.assertFalse(os.path.exists('pylint_logilab_common.txt'))
 
     def test_enable_report(self):
         self.assertEqual(self.linter.report_is_enabled('RP0001'), True)
@@ -381,9 +382,17 @@ class PyLinterTC(TestCase):
 
     def test_init_hooks_called_before_load_plugins(self):
          self.assertRaises(RuntimeError,
-                           Run, ['--load-plugins', 'unexistant', '--init-hooks', 'raise RuntimeError'])
+                           Run, ['--load-plugins', 'unexistant', '--init-hook', 'raise RuntimeError'])
          self.assertRaises(RuntimeError,
-                           Run, ['--init-hooks', 'raise RuntimeError', '--load-plugins', 'unexistant'])
+                           Run, ['--init-hook', 'raise RuntimeError', '--load-plugins', 'unexistant'])
+
+
+    def test_analyze_explicit_script(self):
+        self.linter.set_reporter(TestReporter())
+        self.linter.check(self.datapath('ascript'))
+        self.assertEqual(
+            ['C:  2: Line too long (175/80)'],
+            self.linter.reporter.messages)
 
 class ConfigTC(TestCase):
 
